#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import cv2
import numpy as np
import math
import subprocess
import random

import matplotlib.pyplot as plt

from img_load_tools import h5_to_img, load_h5, load_png
#from draw_result import draw_result

from visualize_tools import visualize


#    # --- USE libsvm
# sys.path.append('lib/libsvm_mod/python/')
# from svmutil import *   # libsvm

# --- USE sklearn
from sklearn.svm import SVC
from sklearn.model_selection import GridSearchCV


SCALE=20


def load_config(dname):
    with open(dname + 'config.txt', 'r') as f:
        config_list = []
        for line in f:
            item_list = line.split()
            config_list.append(item_list[-1])
    return config_list

def load_script(dname):
    with open(dname + 'script.txt', 'r') as f:
        config_list = []
        for line in f:
            item_list = line.split()
            config_list.append(item_list[-1])
    return config_list

def get_seed_name(path, class_num):
    return [path + 'seed' + str(i) + '.png' for i in range(class_num)]

def img_to_long_array(h, w, img_BGR, disparity):
    '''
     Load image and reshape it to [ (cr_0, cb_0), (cr_1, cb_1), ... , (cr_h*w, cb_h*w) ] 
     Returns raw_image 
    '''
    img_YCRCB = cv2.cvtColor(img_BGR, cv2.COLOR_BGR2YCR_CB)
    array_CR = img_YCRCB[:, :, 1].reshape((h * w))
    array_CB = img_YCRCB[:, :, 2].reshape((h * w))

    array_disp = disparity.reshape((h * w))
    return img_YCRCB, img_BGR, array_CR, array_CB, array_disp

def load_seeds(seed_name):
    list_exists = []
    for i in range(class_num):
        print (seed_name[i])

        list_exists.append(np.array(np.ravel(cv2.imread(seed_name[i], -1)[:, :, 3]))) 

    return list_exists

def whitening(x):
    '''
    x <- (x - average) / std
    '''
    return x
    # needed to be shared b/w training and predicting, but not shared. 
    
    '''
    eps = 0.001
    ave = np.average(x)
    std = np.std(x + eps)   # except zero devide

    ave_array = np.ones(x.shape) * ave
    x_ = (x - ave_array) / std

    ave_ = np.average(x_)
    std_ = np.std(x_) 

    print ('>Whitening:', ave, std,'->', ave_, std_)

    return x_
    '''

def make_training_data(use_disparity, feature, list_exists):
    training_feature = []
    training_label   = []

    for i in range(class_num):
        exist = list_exists[i]
        tmp = [f.tolist() for j, f in enumerate(feature) if exist[j] > 0]
        training_feature += tmp
        training_label += ([i for j in range(len(tmp))])

    return training_feature, training_label

'''
def generate_classifier(training_CRCB, training_label):

#     # --- USE libsvm
#     prob = svm_problem(training_label, training_CRCB)
#     param = svm_parameter('-s 0 -t 2')
#     classifier = svm_train(prob, param)
#     #
#     #svm_save_model('model.txt', classifier)
#     return classifier 
    
    # --- USE sklearn
    parameters = [{
        'kernel':['rbf'], 
        'C':np.logspace(-3, 3, 7), 
#         'gamma':np.logspace(-3, 0, 4)
    }]
    clf = GridSearchCV(SVC(probability =True, gamma='scale'), parameters, cv=5, n_jobs = -1)
    clf.fit(training_CRCB, training_label)
    print(clf.grid_scores_)

    return clf.best_estimator_    

def prediction(classifier, test_data, class_num):
#     # --- USE libsvm
#     temp_label = [0 for i in range(len(test_data))]

#     predicted_label, p_acc, p_vals = svm_predict(temp_label, test_data, classifier)

#     #visualize(test_data, predicted_label, use_disparity=1, skip=300)

#     predicted_label = np.array(predicted_label)
#     p_vals_array = np.array(p_vals)
#     return  predicted_label.reshape((h, w)), p_vals_array.reshape((h, w))

    # --- USE sklearn
    p_vals_array = classifier.predict_proba(test_data)[:,1]*100
   
    return  0, p_vals_array.reshape((h, w))
'''

def get_max_p_val(p_vals):
    mat = np.array(p_vals)
    max_p_val = max(abs(mat.max()), abs(mat.min())) 
    print (max_p_val)
    return max_p_val

def normalize(max_p_val, p_val, h, w):
    '''
    Normalize mat 0 to 100
    '''
    p_val = ((p_val / max_p_val) + 1. ) * 50.
    return p_val

def sigmoid(max_p_val, p_val, h, w):
    '''
    trans distance to class posterior prob. using sigmoid function
    '''
    a = 1. / max_p_val  # dicide gradiant
    #sigm = [100. / (1. + math.exp(-a * x) ) for x in p_val.reshape((h * w))]
    sigm = 100. / (1. + np.exp(-a * p_val) ) #for x in p_val.reshape((h * w))]
    print ('Sigmoid ave: ',np.average(np.array(sigm)))
    print ('Sigmoid var: ',np.var(np.array(sigm)))
    return np.array(sigm)


def likelihood(vv, uu, out_path, c, p_vals, path, seed_exists, h, w):

    # set likelihood 0/100 where marker is drawn
    p_vals_ = p_vals.reshape((h * w))
    obj_seed = seed_exists[c]
    mask100 = obj_seed != 0

    p_vals_[mask100] = SCALE

    for j in range(class_num):
        if j != c:
            mask0 = seed_exists[j] != 0
            p_vals_[mask0] = 0
    p_vals = p_vals_.reshape((h, w))

    #plt.imshow(p_vals)
    #plt.colorbar()
    #plt.savefig(path + 'likelihood_graph'+str(c)+'.png')
    #plt.show()

    #subprocess.call(['mkdir' ,path + 'likelihood'])
    oname =  path + 'likelihood/' + str(c) + '_' + str(vv) + '_' + str(uu) + '.png'
    cv2.imwrite(oname, p_vals)

    return p_vals

def _likelihood(out_path, max_p_val, c, p_vals, path, seed_exists, h, w):

    # set likelihood 0/100 where marker is drawn
    p_vals_ = p_vals.reshape((h * w))

    obj_seed = seed_exists[c]

    #for i in range(h * w):
    #    if obj_seed[i] != 0:
    #        p_vals_[i] = 100    # 100% likelihood of class c

    #    for j in range(class_num):
    #        if j != c and seed_exists[j][i] != 0:
    #            p_vals_[i] = 0    # force to 0%

    p_vals = p_vals_.reshape((h, w))

    #plt.imshow(p_vals)
    #plt.colorbar()
    #plt.savefig(path + 'likelihood_graph'+str(c)+'.png')
    #plt.show()

    #subprocess.call(['mkdir' ,out_path + 'likelihood' + str(max_p_val)])
    cv2.imwrite(out_path + 'likelihood'+str(c)+'.png', p_vals)

    return p_vals

def draw_result_using_p_vals(i, path, h, w, class_num, p_val, img_BGR):
    col_result = np.zeros((h, w, 3))
    #color = colors[i]
    img = np.zeros((h, w, 4))
    img[:, :, :3] = img_BGR
    img[:, :, 3][p_val > 50] = 255
    cv2.imwrite(path + str(i) + '.png', img)

    for j, c in enumerate(color):
        if int(c) == 255: 
            col_result[:, :, 1][p_val > 50 ] = 255

    #cv2.imwrite(path + 'result' + str(i) + '.png', col_result)
    #cv2.imshow('win', col_result)
    #cv2.waitKey(10)
    #cv2.destroyAllWindows()

    return  


if __name__ == '__main__':
    path = sys.argv[1]
    config_list = load_config(path)

    # params
    class_num       = int(config_list[0])
    #path            = config_list[1]
    src_type        = bool(int(config_list[1]))
    src_name        = path + config_list[2]
    disp_name       = path + config_list[3]
    w, h   = int(config_list[4]), int(config_list[5])
    use_disparity   = bool(config_list[6])
    #out_path        = config_list[7] 
    out_path        = path
    #colors = [] * class_num

    #if src_type:
    #    img = h5_to_img(path, load_h5(path)) 
    #else:
    #    img = load_png(src_name, disp_name) 
    #img = cv2.imread(path + '2_2.png', 1)
    img = cv2.imread(src_name, 1)

    #disp_img = cv2.imread(path + 'disparity/2_2.png', 0)
    disp_img = cv2.imread(disp_name, 0)

    img_YCRCB, img_BGR, array_CR, array_CB, array_disp = img_to_long_array(
        h, w, img, disp_img)

    array_CR = whitening(array_CR)
    array_CB = whitening(array_CB) 
    array_disp = whitening(array_disp)

    array_CRCB = np.dstack((array_CR, array_CB))

    if use_disparity:
        print ('use disp')
        feature = np.dstack((array_CRCB, array_disp))
    else:
        print ('not use disp')
        feature = array_CRCB

    seed_name   = get_seed_name(path, class_num)
    seed_exists = load_seeds(seed_name)

    training_feature, training_label_org = make_training_data(use_disparity, feature[0][:], seed_exists)

    #visualize(training_feature, training_label, use_disparity, skip=30)

    # max_p_val = 10
    # classifier = [0] * class_num
    # p_vals = [0] * class_num

    parameters = [{
        'kernel':['rbf','linear'], 
        'C':np.logspace(-3, 3, 7), 
        'gamma':np.logspace(-3, 0, 4)
    }]
    clf = GridSearchCV(SVC(probability =True), parameters, iid = False, cv=5, n_jobs = -1)

    # parameter grid search
    SAMPLES = min(1000,len(training_label_org))
    sample = np.array(random.sample(np.hstack((np.array([training_label_org]).T,training_feature)).tolist(),SAMPLES))
    clf.fit(sample[:,1:], sample[:,0])
    print("for samples: score=%.3f"%(clf.best_score_))

    classifier = clf.best_estimator_.fit(training_feature, training_label_org)

    scriptConfig = load_script(path)

    
    subprocess.call(["mkdir", path+"/likelihood"])
    for c in range(class_num):
        subprocess.call(["mkdir", "-p", path+"/out/%d"%(c)])
    
    for vv in range(0,int(scriptConfig[5])):
        for uu in range(0,int(scriptConfig[4])):
            fname = path + str(vv) + '_' + str(uu) + '.png'
            print (fname)
            img = cv2.imread(fname, 1)

            img_YCRCB = cv2.cvtColor(img, cv2.COLOR_BGR2YCR_CB)
            array_CR = img_YCRCB[:, :, 1].reshape((h * w))
            array_CB = img_YCRCB[:, :, 2].reshape((h * w))

            array_CR = whitening(array_CR)
            array_CB = whitening(array_CB)
            array_CRCB = np.dstack((array_CR, array_CB))

            dname = path + 'disparity/' + str(vv) + '_' + str(uu) + '.png'
            disparity = cv2.imread(dname, 0)
            array_disp = disparity.reshape((h * w))
            array_disp = whitening(array_disp)
            feature = np.dstack((array_CRCB, array_disp))

            print ('prediction')
            feature_list = feature[0][:].tolist()
            p_val = classifier.predict_proba(feature_list)*SCALE # should be 100, but for consistency with mihara

            print ('output likelihood')
            for c in range(class_num):
                likelihood(vv, uu, out_path, c, p_val[:,c], path, seed_exists, h, w)
            #draw_result_using_p_vals(c, path, h, w, class_num, p_val, img_BGR, colors)
