import numpy as np

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D 

def visualize3D(training_CRCB, training_label, skip):
    '''
    visualize 3D feature space
    '''

    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')

    CR = np.array(zip(*training_CRCB)[0])
    CB = np.array(zip(*training_CRCB)[1])
    D  = np.array(zip(*training_CRCB)[2])
    training_label = np.array(training_label)

    CR = CR[::skip]
    CB = CB[::skip]
    D  =  D[::skip]
    training_label = training_label[::skip]

    ax.scatter(CR, CB, D, c=training_label, cmap=plt.cm.Paired)

    ax.set_xlabel('CR')
    ax.set_ylabel('CB')
    ax.set_zlabel('Disparity')

    plt.show()

def visualize2D(training_CRCB, training_label, skip):
    '''
    visualize 2D feature space

    --NOTE--
     when a list = [(1, 2), (3, 4), (5, 6)] is given,
     zip(*list) returns [(1, 3, 5), (2, 4, 6)] 
    '''
    CR = zip(*training_CRCB)[0]
    CB = zip(*training_CRCB)[1]

    CR = CR[::skip]
    CB = CB[::skip]

    plt.scatter(x=CR, y=CB, c=training_label, cmap=plt.cm.Paired)
    plt.show()

def visualize(training_data, training_label, use_disparity, skip):
    if use_disparity:
        visualize3D(training_data, training_label, skip)
    else:
        visualize2D(training_data, training_label, skip)
    return

if __name__ == '__main__':
    print('main!')
