import numpy as np
import cv2
import subprocess
import sys

path        = sys.argv[1]
ex_path     = path + 'out/'
GT_path     = path + 'GT/'
diff_path   = path + 'diff/'

num = 5      # angular resolution of light-field
lnum = int(sys.argv[2])

def evaluate():
    '''
    Get segmentation accuracy via counting non-zero pixels of diff image.
    '''
    acc_sum = 0. # initialize accuracy
    
    acc_list = []

    f = open(path + 'result.txt', 'w')
    for j in range(num):
        for i in range(num):
            diff_name = diff_path + str(j) + "_" + str(i) + '.png'
            diff = cv2.imread(diff_name, 0)
            S = diff.shape[0] * diff.shape[1] 
            non_zero = diff != 0

            acc = 100 - float(np.nonzero(non_zero)[0].size )  / float(S) * 100.
            print  str(j) + "_" + str(i), 'acc:', acc
            f.write( str(j) + "_" + str(i) + 'acc: ' + str(acc) + '\n')
            acc_list.append(acc)

            acc_sum += acc

    print 'result:', acc_sum / float(num * num)
    f.write( 'result: ' + str( acc_sum / float(num * num) )  + '\n')
    print 'variance:', np.var(np.array(acc_list))
    f.close()

    f = open('final_result.txt', 'a') 
    f.write( str( acc_sum / float(num * num) )  + '\n' )
    f.close()
    return


def colorize():
    #num = 5
    for p1 in range(5):
        for p2 in range(5):
            print ex_path + '0/' + str(p1) + '_' + str(p2) + '.png'
            label0 = cv2.imread(ex_path + '0/' + str(p1) + '_' + str(p2) + '.png', -1)[:, :, 3]
            label1 = cv2.imread(ex_path + '1/' + str(p1) + '_' + str(p2) + '.png', -1)[:, :, 3]
            label2 = cv2.imread(ex_path + '2/' + str(p1) + '_' + str(p2) + '.png', -1)[:, :, 3]
            label3 = cv2.imread(ex_path + '3/' + str(p1) + '_' + str(p2) + '.png', -1)[:, :, 3]
            label4 = cv2.imread(ex_path + '4/' + str(p1) + '_' + str(p2) + '.png', -1)[:, :, 3]
            label5 = cv2.imread(ex_path + '5/' + str(p1) + '_' + str(p2) + '.png', -1)[:, :, 3]

            result = np.zeros((label0.shape[0], label0.shape[1], 3))


            result[:, :, 2][ label0 == 255 ] = 255  #R

            result[:, :, 0][ label1 == 255 ] = 255  #R

            result[:, :, 1][ label2 == 255 ] = 255  #G
            result[:, :, 2][ label2 == 255 ] = 255  #G

            result[:, :, 1][ label3 == 255 ] = 255  #Y
            result[:, :, 0][ label3 == 255 ] = 255  #Y 

            result[:, :, 2][ label4 == 255 ] = 255  #Y
            result[:, :, 0][ label4 == 255 ] = 255  #Y 

            result[:, :, 0][ label5 == 255 ] = 128  #Y
            result[:, :, 1][ label5 == 255 ] = 128  #Y
            result[:, :, 2][ label5 == 255 ] = 128  #Y

            cv2.imwrite(ex_path + "c_" + str(p1) +'_'+ str(p2) + '.png', result)
    return

def label_assignment():
    path =  sys.argv[1] + "out/"


    label = [] * lnum

    h, w, c = cv2.imread(path + '0/0_0.png', 1).shape
    for p1 in range(num):
        for p2 in range(num):
            result = np.zeros((h, w))

            for l in range(lnum):
                fname = path + str(l) + '/' + str(p1) + '_' + str(p2) + '.png'
                label_img = cv2.imread(fname, -1)[:, :, 3]

                result[:, :][ label_img != 0 ] = l + 1

            cv2.imwrite(path + str(p1) +'_'+ str(p2) + '.png', result * 10) 

def diff():
    """
    Make diff with ground truth and experimental segmentation result 
    Diff image is made by "composite -compose Difference img1 img2 outname" on ImageMagick.
    """

    subprocess.call(['mkdir', diff_path])
    
    for j in range(num):
        for i in range(num):
            diff_name = diff_path + str(j) + "_" + str(i) + '.png'
            ex_img = ex_path + str(j) + "_" + str(i) + ".png"
            gt_img = GT_path + str(j) + "_" + str(i) + ".png" 
            subprocess.call(['composite', '-compose', 'Difference', gt_img, ex_img, diff_name]) 

if __name__ == '__main__':
    diff()
    label_assignment()
    #colorize()
    evaluate()
