#!/usr/bin/python
# -*- coding: utf-8 -*-

from PyQt4 import QtGui, QtCore
import numpy as np
import cv2
import sys
import subprocess
import os.path

path_aSeg = './aSeg.py' # path to aSeg.py

# files
#fname_in = './src/1_1.png' # central image (input)
fname_in = sys.argv[1] # central image (input)
#fname_out = './user_input_mask.png' # drawn mask (output, seed of grabcut)
fname_gc_result = './preview.png' # result of grabcut (input for preview)

disp_scale = 1.5    # scale of displayed image
disp_gamma = 0.6
screen_width = 1200

cap_view_paint = u'領域選択'
cap_view_preview = u'プレビュー'
cap_view_control = u'操作'
cap_button_undo = u'Foreground'
cap_button_save = u'Background'
cap_confirm_grabcut = u'物体選択の確認'
msg_confirm_grabcut = u'消したい部分が黒くなっていますか?'
cap_confirm_grabcut_failed = u'エラー'
msg_confirm_grabcut_failed = u'グラブカットに失敗しました\nマスクが書かれていません'



class paintScene(QtGui.QGraphicsScene):
    def __init__(self, *argv, **keywords):
        self.brush_size = 3    # default brush size
        self.image_scale = 1  # image scale

        self.BRUSH_SIZE_MIN = 3.
        self.BRUSH_SIZE_MAX = 200.
        self.BRUSH_COLOR_FILL = QtGui.QColor(255, 0, 0, 127)
        self.BRUSH_COLOR_DRAW = QtGui.QColor(0, 0, 255, 127)

        super(paintScene, self).__init__(*argv, **keywords)

        # set up brush cursor
        self.ellipse_cursor = QtGui.QGraphicsEllipseItem(0, 0, self.brush_size, self.brush_size)
        self.ellipse_cursor.translate(-self.brush_size/2, -self.brush_size/2)
        pen_cursor = QtGui.QPen(self.BRUSH_COLOR_DRAW)
        brush_cursor = QtGui.QBrush(self.BRUSH_COLOR_FILL)
        self.ellipse_cursor.setPen(pen_cursor)
        self.ellipse_cursor.setBrush(brush_cursor)
        self.ellipse_cursor.setZValue(2)
        self.addItem(self.ellipse_cursor)

        # set up brush strokes 
        self.stroke_list = []
        self.changeBrushSize(self.brush_size)

        self.STATE_PAINTING = False
        self.PREVIEW_MODE = False

    def setPreviewMode(self, preview_mode):
        self.PREVIEW_MODE = preview_mode

        if preview_mode:
            self.image_item.setZValue(3)
        else:
            self.image_item.setZValue(0)

    def changeBrushSize(self, size):
        self.brush_size = size
        self.changeCursorSize(size)

    def changeCursorSize(self, size):
        old_cursor_rect = self.ellipse_cursor.rect()
        self.ellipse_cursor.translate(old_cursor_rect.width()/2, old_cursor_rect.width()/2)
        self.ellipse_cursor.setRect(old_cursor_rect.left(), old_cursor_rect.top(), size, size)
        self.ellipse_cursor.translate(-size/2, -size/2)

    def addPathItem(self, size):
        brush_path = QtGui.QPainterPath()
        brush_path_item = QtGui.QGraphicsPathItem(brush_path)
        pen = QtGui.QPen(self.BRUSH_COLOR_FILL)
        pen.setWidth(self.brush_size)
        pen.setCapStyle(QtCore.Qt.RoundCap)
        brush_path_item.setPen(pen)
        brush_path_item.setZValue(1)

        self.stroke_list.append((brush_path, brush_path_item))
        self.addItem(brush_path_item)
        return brush_path

    def setImage(self, image):
        self.img_size_org = image.size()
        self.img_size = QtCore.QSize(image.size().width()*disp_scale,
                                     image.size().height()*disp_scale)
        image = image.scaled(self.img_size.width(), self.img_size.height())
        self.image_item = QtGui.QGraphicsPixmapItem(QtGui.QPixmap.fromImage(image))
        # draw background image
        self.image_item.setZValue(0)
        self.image_item.setPos(0, 0)
        self.addItem(self.image_item)

    def getMask(self):
        bmp_mask = QtGui.QPixmap(self.img_size)
        #bmp_mask.fill(QtGui.QColor(0, 0, 0, 0))
        bmp_mask.fill(QtGui.QColor(255, 0, 0, 0))

        # color of mask is green
        pen = QtGui.QPen(QtGui.QColor(0, 255, 0, 255))
        pen.setCapStyle(QtCore.Qt.RoundCap)

        qp = QtGui.QPainter(bmp_mask)
        qp.setBackground(QtGui.QBrush(QtGui.QColor(255, 0, 0, 0)))
        for brush_path, brush_path_item in self.stroke_list:
            pen.setWidth(brush_path_item.pen().width())
            qp.setPen(pen)
            qp.drawPath(brush_path)
        qp.end()
        bmp_mask = bmp_mask.scaled(self.img_size_org.width(), 
                                   self.img_size_org.height())
        return bmp_mask.toImage()

    def undoStroke(self):
        if self.PREVIEW_MODE:
            return
        # erase stroke if right button was clicked
        if len(self.stroke_list) == 0:
            return
        self.removeItem(self.stroke_list[-1][1])
        del self.stroke_list[-1]
        self.changeBrushSize(self.brush_size)

    def mousePressEvent(self, event):
        if self.PREVIEW_MODE:
            return
        if event.button() == QtCore.Qt.LeftButton:
            # start drawing
            brush_path = self.addPathItem(self.brush_size)
            cur_pos = event.scenePos()
            brush_path.moveTo(cur_pos.x(), cur_pos.y())
            self.STATE_PAINTING = True
        elif event.button() == QtCore.Qt.RightButton:
            self.undoStroke()

    def mouseMoveEvent(self, event):
        if self.PREVIEW_MODE:
            return
        # move cursor
        cur_pos = event.scenePos()
        self.ellipse_cursor.setPos(cur_pos.x(), cur_pos.y())

        if self.STATE_PAINTING:
            # extend path
            brush_path, brush_path_item = self.stroke_list[-1]
            brush_path.lineTo(cur_pos.x(), cur_pos.y())
            brush_path_item.setPath(brush_path)

    def mouseReleaseEvent(self, event):
        if self.PREVIEW_MODE:
            return
        if event.button() == QtCore.Qt.LeftButton:
            self.STATE_PAINTING = False

    def wheelEvent(self,event):
        if self.PREVIEW_MODE:
            return
        if self.STATE_PAINTING == True:
            return
        # change brush size by wheel
        self.brush_size += event.delta() / 10
        # limit brush size
        if self.brush_size < self.BRUSH_SIZE_MIN:
            self.brush_size = self.BRUSH_SIZE_MIN
        if self.brush_size > self.BRUSH_SIZE_MAX:
            self.brush_size = self.BRUSH_SIZE_MAX
        self.changeBrushSize(self.brush_size)


class paintView(QtGui.QGraphicsView):
    def __init__(self):
        super(paintView, self).__init__()

        self.setCacheMode(QtGui.QGraphicsView.CacheBackground)
        self.setRenderHints(QtGui.QPainter.Antialiasing |
            QtGui.QPainter.SmoothPixmapTransform |
            QtGui.QPainter.TextAntialiasing )
        self.setMouseTracking(True)
        self.setNewScene()
        self.move(0, 0)

    def setNewScene(self):
        self.scene = paintScene(self)
        self.scene.setSceneRect(QtCore.QRectF(self.rect()))
        self.setScene(self.scene)
        self.setWindowTitle(cap_view_paint)

    def setImage(self, image):
        # gamma correction
        np_image = np.uint8(cv2.pow(image/255.0, disp_gamma) * 255.)

        # convert and set image
        qimg_in = nparrayToQimg(np_image)
        self.qimg_in = qimg_in
        self.scene.setImage(qimg_in)
        r = qimg_in.rect()
        # adjust window size
        self.setSceneRect(QtCore.QRectF(0, 0,
                           r.right()*disp_scale, r.bottom()*disp_scale))
        win_size = QtCore.QSize(r.right()*disp_scale, r.bottom()*disp_scale)
        self.setFixedSize(win_size)
        # off scrollbars
        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)

    def getScene(self):
        return self.scene

    def onUndoButtonClicked(self):
        saveMask(self.scene, 'seed_fg.png')
        self.scene.undoStroke()

    def onSaveButtonClicked(self):
        saveMask(self.scene, 'seed_bg.png')
        self.scene.undoStroke()
        
    def closeEvent(self, event):
        exit()


class interfaceView(QtGui.QGraphicsView):
    def __init__(self):
        self.CONTROL_WIDTH = 250
        self.CONTROL_HEIGHT = 100

        super(interfaceView, self).__init__()

        self.setCacheMode(QtGui.QGraphicsView.CacheBackground)
        self.setRenderHints(QtGui.QPainter.Antialiasing |
            QtGui.QPainter.SmoothPixmapTransform |
            QtGui.QPainter.TextAntialiasing )

        # build UI
        self.setFixedSize(QtCore.QSize(self.CONTROL_WIDTH, self.CONTROL_HEIGHT*2))
        self.setGeometry(screen_width - self.CONTROL_WIDTH, 0,
            self.CONTROL_WIDTH, self.CONTROL_HEIGHT)

        self.setWindowTitle(cap_view_control)
        self.button_undo = QtGui.QPushButton(cap_button_undo, self)
        self.button_save = QtGui.QPushButton(cap_button_save, self)
        self.button_undo.setFixedSize(self.CONTROL_WIDTH, self.CONTROL_HEIGHT)
        self.button_save.setFixedSize(self.CONTROL_WIDTH, self.CONTROL_HEIGHT)
        self.button_save.move(0, self.CONTROL_HEIGHT)

    def setPaintView(self, paint_view):
        self.paint_view = paint_view
        # set window position
        paint_pos = paint_view.rect().bottomRight()
        global_point = self.mapToGlobal(paint_pos)
        self.move(global_point.x() + paint_view.width(), global_point.y())
        # link button to function
        self.button_undo.clicked.connect(paint_view.onUndoButtonClicked)
        self.button_save.clicked.connect(paint_view.onSaveButtonClicked)

    def closeEvent(self, event):
        exit()


def nparrayToQimg(array_bgr):
    array_rgb = array_bgr.copy()
# swap R and B channel
    array_rgb[:,:,0], array_rgb[:,:,2] = array_bgr[:,:,2], array_bgr[:,:,0]
    height, width, dim = array_bgr.shape
    bytes_per_line = dim * width
    qimg = QtGui.QImage(array_rgb.data, width, height, bytes_per_line, QtGui.QImage.Format_RGB888)
    return qimg

def saveMask(scene, fname_out):
    print 'Saving mask'
    qimg = scene.getMask()

    # convert qpixmap to numpy array
    qimg = qimg.convertToFormat(QtGui.QImage.Format_RGB32)
    width = qimg.width()
    height = qimg.height()
    ptr = qimg.bits()
    ptr.setsize(qimg.byteCount())
    img_mask = np.array(ptr).reshape(height, width, 4)  #  Copies the data

    # merge original image and alpha channel
    result = np.copy(img_mask)
    result[:, :, 3] = img_mask[:, :, 1]
    result[:, :, 0:3] = np_img_in[:, :, 0:3]
    # write to file
    cv2.imwrite(fname_out, result)
    print 'save', fname_out
    return

def callGrabcut():
    print 'Call grabcut'
    subprocess.call(["python", path_aSeg, fname_in, fname_out])
    if not os.path.exists(fname_gc_result):
        print 'Grabcut failed'
        return None
    else:
        print 'Grabcut finished'
    img = cv2.imread(fname_gc_result)
    #qimg = nparrayToQimg(img)
    return img


# override input file name, if argument is set
if len(sys.argv) == 2:
    fname_in = sys.argv[1]
    print 'Input file given'
if not os.path.exists(fname_in):
    print 'Couldn\'t open ' + fname_in
    exit()
else:
    print 'Input file: ' + fname_in

app = QtGui.QApplication(sys.argv)
# read central image
np_img_in = cv2.imread(fname_in)

# make window
paint_view = paintView()
paint_view.setImage(np_img_in)
interface_view = interfaceView()
interface_view.setPaintView(paint_view)
paint_view.show()
interface_view.show()

sys.exit(app.exec_())
